      subroutine gdist(dlaA,dloA,dlaB,dloB,azis,azir,range)

c     this routine is a version of George Dworski's iGEOD.f program
c     that geodesics between two points on the earth`s surface.

c     this version was designed specifically to replace Bruce Cornuelle's
c     dist.f routine.  the main differences are that this calculates
c     geodesics more accurately (to the WGS84 ellipsoid) and that the
c     azir is no longer orientated to point back to the source--it 
c     points forward.

c     input:two lat/long pairs
c     output:two azimuths at the points and the distance between the points.

c     This routine takes the (input) parameters from the subroutine call
c     rather than interactively.  Many of the options available on the
c     interactive version (iGEOD.f), such as degree/min/sec input, 
c     output of GREAT CIRCLE calculations and other information
c     have been eliminated.  

c                                                craig friesen 7/92

c========================================================================      
c     real*8: ITERATIVELY DETERMINES GEODESIC "A"-to-"B"
c     System of 2+1 GEODESIC O.D.E. (for lat, long, and azimuth)
c     driver for the /Drkf45/,/Drkfs/, and /Dfehl/ integr. group,
c     ellipsoidal earth GEODESIC version using /Dfff,/DgeordG.


c     Iterative loop on geodesics :
c     s       = distance  [km]
c     y(1)    = latitude  [rad]
c     y(2)    = longitude [rad]
c     y(3)    = azimuth   [rad] (positive cwse from North)
c     yp(i)   = (first) time derivatives of /y/ above, i=1,2,3
c       remark... this program must provide
c                  (in addition to error and output management) :
c               (1) initial conditions on vector /y/
c               (2) initial distance in /s/ (typically 0.)

c     ==========================================================
      implicit real*8 (a-h,o-z)

      external fff

      real*8 r0,pi,torad,dloBA,aloBA,deps
      real*8 dlaA,dloA,dlaB,dloB
      real*8 daxc,daxs,dbxc,dbxs,dccxc,dccxs,dcxc,dcxs
      real*8 cosaa,sinaa,cosbb,sinbb
      real*8 da,db,dc,daa,dbb,dcc
      parameter( pi=3.141592653589d0,pi2=2.d0*pi )
      parameter( torad=1.745329252d-02 )
      parameter( r0=6.371005d+03 )
      parameter( s0=0.d0,sout1=0.d0,dels=100.d0 )
      parameter( neqn=3 )
      real*8 fk0,glat,cosl,rpve,dsina,ddummy
      real*8  azdc,cosc,dpi
      real*8 s,uIC(neqn),u(neqn),sout,relerr,abserr
      real*8 work(21)
      integer           iwork(5),iflag
      character*6       lead,blank

      data     lead/'  GEOD'/,blank/'     '/
      data     deps/5.d-4/

  501 format(/)
      
      if ((ABS (dlaA - dlaB) + ABS (dloA - dloB)) .lt. 5.0d-10) then
         azis = 0.d0
         azir = 0.d0
         range = 0.d0
         return
      endif

      dels0 = 1000.d0

c     make WESTERN HEMISPHERE longitudes to be NEGATIVE
      if (dloA.gt.180.) dloA=dloA-360.d0
      if (dloB.gt.180.) dloB=dloB-360.d0
      
c     compute  CO-LATITUDES
      da    =  (90.d0 - dlaB)  * torad
      db    =  (90.d0 - dlaA)  * torad

c     and compute LONGITUDE DIFFERENCE
      dloBA =   dloB  - dloA

c     restrict longitude difference to interval +/- 180 degrees
      if     (dloBA.gt. 180.d0) then
        dloBA = dloBA - 360.d0
      elseif (dloBA.lt.-180.d0) then
        dloBA = dloBA + 360.d0
      endif
c     and then consider only the absolute value of the polar angle
      aloBA   = abs(dloBA)

c     Oblique spherical triangle :
c     ----------------------------
      dcc   =  dabs(dloBA) * torad
      daxc  =  dcos(da)
      daxs  =  dsin(da)
      dbxc  =  dcos(db)
      dbxs  =  dsin(db)
      dccxc =  dcos(dcc)
      dccxs =  dsin(dcc)
      dcxc  =  daxc*dbxc + daxs*dbxs*dccxc
      if (dcxc.le.-1.d0) then
        dc  =  pi
      else
        dc  =  dacos(dcxc)
      endif
      dcxs  =  dsin(dc)
      if (dcxs.lt.1.d-8) then
        cosaa = 1.d0
        sinaa = 0.d0
        cosbb = 1.d0
        sinbb = 0.d0
      else
        cosaa = (daxc-dbxc*dcxc)/(dbxs*dcxs)
        sinaa =  dccxs/dcxs*daxs
        cosbb = (dbxc-daxc*dcxc)/(daxs*dcxs)
        sinbb =  dccxs/dcxs*dbxs
      endif
      daa   =  datan2(sinaa,cosaa)
      dbb   =  datan2(sinbb,cosbb)
c     cludge :
      daaanom = 180.d0-abs(daa)/torad
      dbbanom = 180.d0-abs(dbb)/torad
      if (daaanom.lt.0.02d0) daa=abs(daa)
      if (dbbanom.lt.0.02d0) dbb=abs(dbb)

      cgc   =  real(dc/torad)
      ss    =  real(dc*r0)
c     ----------------------------

      if (dloBA.ge.0.d0) then
        azA =  real(         daa /torad)
        azB =  real((2.d0*pi-dbb)/torad)
      else
        azA =  real((2.d0*pi-daa)/torad)
        azB =  real((        dbb)/torad)
      endif
      if (azB.ge.180.d0) then
        azB = azB-180.d0
      else
        azB = azB+180.d0
      endif


      dpi    =dble(pi)
      abserr0=1.5d-9
      relerr0=1.0d-9


      niter  = -1
      azim0  = azA
      if (abs(azim0-180.d0).lt.0.01d0 .and. ss.gt.2.d4) then
        write(0,*) ' SORRY, geodesic heading across S.Pole'
        write(0,*) '        apply singular analysis'
        goto 050
      elseif (abs(azim0)   .lt.0.01d0 .and. ss.gt.2.d4) then
        write(0,*) ' SORRY, geodesic heading across N.Pole'
        write(0,*) '        apply singular analysis'
        goto 050
      endif

      sfin0  = ss
c     REMARK: integration is based on path-length /s
      sfinal = sfin0
      chzold = 0.d0
c-      disold = 0.d0
      nitlim = 100
      nsam   = 0

c     ITERATIVE SEARCH LOOP :
  010 continue
      niter = niter + 1

      abserr=abserr0
      relerr=relerr0

c     IC => initial conditions [rad] : lat(A),lon(A), and azimuth(at A)
c                 ( reference: [rad] : lat(B),lon(B))
      dlaAr     =  dlaA *torad
      dloAr     =  dloA *torad
      dlaBr     =  dlaB *torad
      dloBr     =  dloB *torad
      uIC(1)    =  dlaAr
      uIC(2)    =  dloAr
      uIC(3)    =  azim0*torad
      u(1)      =  uIC(1)
      u(2)      =  uIC(2)
      u(3)      =  uIC(3)

c     define Clairaut's invariant for this geodesic :
      glat      =  dble(uIC(1))
      dsina     =  dble(sin(uIC(3)))
c     *******************************************
      call geordG( glat,ddummy,cosl,ddummy,rpve )
c     *******************************************
      fk0       =  rpve*cosl*dsina

c     INITIALIZE O.D.E. SOLVER AT sout = s = s0 = 0.
      s         =  s0
      sout      =  s
      iflag     = +1
c     ************************************************************
      call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      nst       =  1
      if (iflag.gt.2) then
        write (0,*)'WARNING: after 1-st step: iflag= ',iflag
        write (0,*)' relerr,abserr = ',relerr,abserr
        DO 012 i=1,21
        write (0,*)' i,work : ',i,work(i)
  012   CONTINUE
        DO 014 i=1,5
        write (0,*)' i,iwork: ',i,iwork(i)
  014   CONTINUE
        write (0,*)' RE-SET to +2'
        iflag   = +2
        print *, 'DEBUG gdist.f:do we ever get here?'
      endif
      if (sout1.gt.0.) then
        sout      = sout1
c       ************************************************************
        call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c       ************************************************************
        if (iflag.gt.2)  goto 030
      endif

c     ---------------------
  020 sout      = s + dels
c     ---------------------
      if (sout.gt.sfinal) then
        sout    = sfinal
      endif
  025 nst = nst + 1

c     it is also assumed that the iflag = +2 => normal integration
c     ************************************************************
      call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      if (iflag.gt.2) then
        iflin  = iflag
c       *************************************************
        call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c       *************************************************
        if (iflout.eq.9)  goto 040
        iflag  = iflout
      else
        relerr = relerr0
      endif

      if (s.le.sfinal) then
        if (iflag.gt.2)  goto 030
        if (s.lt.sfinal) goto 020
      endif

c     THE n-th ITERATE HAS BEEN COMPUTED :
      if (s.ge.sfinal) then

c       EVALUATE NEED FOR ADDITIONAL ITERATES :
c      (a) mark terminal point as origin of local scheme :
c                ************
         ufx1  = xprinc(u(2))
c                ************
         ufy1  = u(1)
         cosy1 = cos(ufy1)
         ufazE = u(3)
         azimE = ufazE/torad
c      (b) express "receiver" point"B" relative to origin :
         x0    = dloBr - ufx1
         if (abs(x0).gt.0.2d0) then
           x0  = x0 - pi2*sign(1.d0,x0)
         endif
         y0    = dlaBr - ufy1
c        expressed in [km] (with terminal point as (0,0)) :
         x0    = r0*x0*cosy1
         y0 = r0*y0


c      (c) def: unit"tangent"vector (1km long) to the geodesic at (0,0) :
c        convert azimuth into "mathematical ccwse" angles (0,+/-pi)
c                *********************
         alphE = xprinc(pi/2.d0-ufazE)
c                *********************
         x2    =   cos(alphE)
         y2    =   sin(alphE)
c      (d) inner product: X0dotX2 : projection onto tangent:
         proj  =   x2*x0 + y2*y0
c      (e) outer product: X0crossppX2 : distance from tangent:
         dist  =   x0*y2 - x2*y0
         chazi = dist/sfinal


       if ((abs(proj).lt.deps .and. abs(dist).lt.deps)
     1  .or. niter.gt.nitlim) then
         goto 040
       endif
        sfinal = sfinal + proj

        if (chzold*chazi.ge.0.d0) then
          nsam=nsam+1
        else
          nsam=0
        endif
        if     (nsam.le.3) then
          cfac=1.0d0
        elseif (nsam.eq.0) then
          cfac=0.8d0*cfac
        else
          cfac=1.2d0*cfac
        endif
        chazi  = cfac*chazi
        azim0  = azim0  + chazi/torad
        chzold = chazi

c       DIAGNOSTIC ACCOUNT OF ITERATION :
c        WRITE(15,560) niter,proj,dist,cfac,chazi

c-        if (abs(abs(disold/dist)-1.d0).lt.1.d-2) goto 040
c-        disold = dist


        goto 010
      endif

  030 continue

c     branching to diagnostics of abnormal performance :
      iflin = iflag
c     *************************************************
      call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c     *************************************************
      if (iflout.eq.9)  goto 040
      iflag = iflout
      goto 020

  040 continue
c       use Clairaut's invariant /fk0 as check on correct azimuth
        glat      =  dble(u(1))
c       *******************************************
        call geordG( glat,ddummy,cosl,ddummy,rpve )
c       *******************************************
        azdc    = asin(fk0/(rpve*cosl))
        cosc    = cos(dble(u(3)))
        if (cosc .lt. 0.d0) then
          azdc  = dpi - azdc
        endif
        if (azdc .lt. 0.d0) then
          azdc  = 2.d0*dpi + azdc
        endif
        azic    = dble(azdc)
c       endrun diagnostic :
c     write(6,*)' '
c     write(  6,*)' Convergence END results :'
c     write(  6,*)' Dtang[km] (along  path) : ',proj
c     write(  6,*)' Dnorm[km] (across path) : ',dist
c     write(  6,*)' precision[km] toler     : ',deps
c     write(  6,*)' iterates used #         : ',niter
c     write(  6,507)
c     write(  6,512) proj,dist,niter
c     write(  6,508)
c     write(  6,555) s0,dlaA,dloA,azim0
c     write(  6,555) s,(u(i)/torad,i=1,neqn),azic/torad


        azimr = azic/torad
        
c     we need these to be in meters and radians:

        range = s * 1000.d0
        azis = pi2 * azim0/360.d0 
        azir = pi2 * azimr/360.d0 

c        print*,s,azim0,azimr
        return

  050 continue
      return
c      stop
      end

