
      program irefSOF

Cf  =  Craig Friesen's comments

Cf    CHANGE: 10/93  changed longitude output to 0 to 360 format

Cf    CHANGE: 4/2/93 to non-interactive, command line parameter
Cf    passing version.  Interactive portions suppressed.
Cf    MENU file driver eliminated.

c     DOUB.PREC.: batch compute refracted SOFAR paths w/ init.azimuth
c     System of 2+1 O.D.E. (for lat, long, and azimuth)
c     driver for the /Drkf45/,/Drkfs/, and /Dfehl/ integr. group,
c     ellipsoidal earth SOFAR Fermat path using /Dffr,/DgeordG.

c     prog/irefSOF   status :         18 March 1993

c       J. G. Dworski

c       H a n d l i n g
c       ===============

c     Runs in a open loop reading a menu from the file
c     generically called MENU (a list directed read):
c     LAT[deg], LON[deg], AZIM[deg], STEP[km], RANGE[km], 1OUT[km]
c     ( 1OUT is first output, typically 0, but can be much larger )

c     Latitude and Longitude refers to the TRANSMITTER co-ordinates;
c     Azimuth must be non-negative, clockwise from North,
c     and Range means the total distance of the path.
c     Step refers to the desired output (incremental) range.

c     For each new entry in the MENU (e.g., a new transmit point,
c     or even just a new launch azimuth at the existing transmit
c     point) a new UNIX output file will be written. The first
c     output file will be named
c     fort.11 , and thereafter others will be: fort.12, fort.13, etc.

c     Support reqr'd : /Drkf45,/Drkfs,/Dfehl,/Dffr,/DgeordG
c     n o t e : the corresponding "great circle" version (dr_16.f)
c               substitutes (for /ffr or fff,geordG): ggg,geordS

c       s       = distance  [km]
c       y(1)    = latitude  [rad]
c       y(2)    = longitude [rad]
c       y(3)    = azimuth   [rad] (positive cwse from North)

c       yp(i)   = (first) time derivatives of /y/ above, i=1,2,3

c       remark... this program must provide
c                  (in addition to error and output management) :
c               (1) initial conditions on vector /y/
c               (2) initial distance in /s/ (typically 0.)

c     ==========================================================

      implicit double precision (a-h,o-z)
      parameter( neqn=3,imax=180,jmax=360 )
      parameter( torad=1.745329252d-02 )
      parameter( pi=3.141592653589d0,pi2=2.d0*pi )
      parameter( r0=6.371005d+03,s0=0.d0)
      real  zmat(imax,jmax)
      double precision  dpi
      real*8 lat,long,azi,inc,range, startr, outlong
      double precision  s,uIC(neqn),u(neqn),sout,relerr,abserr
      double precision  work(21)
      integer  iwork(5),iflag
c      character*72 outfil
      character*100 FILE
      character*40 strTemp
      character*6  blank
cf      logical      yes
c      logical      usec

      common/same/    i1old,j1old
      common/datmat/  zmat
c       this is a (imax X jmax) matrix, to be shared with /lapatch.f

      external ffr

      data        blank/'     '/

  501 format(/)
  505 format(a)
  508 format(t8'RANGE't19'LAT't32'LON't44'AZI'/
     1       t8'[km]'t18'[deg]'t31'[deg]'t43'[deg]')
  510 format(f12.3,3f12.6,i5)
 1510 format(f12,3f12.6,i5)
  555 FORMAT(f12.3,3f12.6)

        m = iargc()
        if ((m .ne. 5) .AND. (m .ne. 6)) then 
           write (0,*)'wrong number of arguments.'
           write (0,*)'Usage: gshoot lat(origin) long(origin)',
     *          ' azimuth(origin) increment-range  range <start range>'
c           write (0,*)
           STOP
        endif
        call getarg(1, strTemp)
        read (strTemp, *) lat
        call getarg(2, strTemp)
        read (strTemp, *) long
        call getarg(3, strTemp)
        read (strTemp, *) azi
        call getarg(4, strTemp)
        read (strTemp, *) inc
        call getarg(5, strTemp)
        read (strTemp, *) range
        if (inc .lt. 5.d-1) then
           write (0,*)'increment too small'
c           write (0,*)
           STOP
        endif
        if (m .ne. 5) then 
           call getarg(6, strTemp)
           read (strTemp, *) startr
        else
           startr = 0.d0
        endif

cf      print 501
cf      print*,' a horizontally refracted SOFAR trajectory run started'
cf      print*,' WGS84 (earth ellipsoid) + SOFAR data base > fort.11,...'
cf      print*,'  '
cf      print*,' in the working directory, you must have ...'
cf      print*,' input: a list directed read from file called MENU, w/'
cf      print*,' LAT[deg],LON[deg],AZIM[deg],STEP[km],RANGE[km],1OUT[km]'
cf      print*,' '
cf      call system(' ls -l MENU ')
cf      call system(' cat MENU ')
cf      print*,' continue ?  [CR=yes]'
cf      if (.not.yes(1)) then
cf        goto 999
cf      endif
cf      print*,' '

c     SOUND SPEED FIELD :

c     input: minimum sound speed file "axisvel" (from ~/LEVITyear) must
c     have been copied into the working directory  :
cf      call system(' ls -l axisvel ')
cf      print*,' "axisvel" must be present; continue ?  [CR=yes]'
cf      if (.not.yes(1)) then
cf        goto 999
cf      endif

cf      print*,' '
c     (180 words/record, 360 records)
      len1 = imax*4
cUSER the following line must contain the correct path for the 
cUSER refracted geodesics package:
      CALL GETENV('ATOC_LIBRARY', FILE)
      OPEN(3,file=FILE(1:lnblnk(FILE))//'/axisvel',form='unformatted',
     1     access='direct',recl=len1)
      do 040 jm = 1,jmax
c       ---------------------------------------------
        read(3,rec=jm,END=950) (zmat(i,jm),i=1,imax)
c       ---------------------------------------------
  040 continue
      i1old = 1
      j1old = 1
      CLOSE(3)

      dpi    =dble(pi)
      abserr0=1.5d-9
      relerr0=1.0d-9
cf      luo    = 10

cf      print*,' by default ... '
cf      print*,' absolute error = ',abserr0
cf  080 print*,' relative error = ',relerr0
cf      print*,' O.K.?  [CR=yes]'
cf      if (.not.yes(1)) then
cf        print*,' ENTER new relative error :'
cf        print*,' (we do not change the absolute error)'
cf        read*, relerr0
cf        goto 080
cf      endif

cf      OPEN(7,file='MENU')
cf  100 read(7,*,END=250) flat0,flon0,azim0,dels0,sfin0,sout1

      flat0 = lat
      flon0 = long
      azim0 = azi
      dels0 = inc / 1000
      sfin0 = range / 1000
      sout1 = startr / 1000
      

cf      write(  6,501)
cf      write(  6,*)' NEW'
cf      write(  6,*)' Transm. from lat,lon [deg]: ',flat0,flon0
cf      write(  6,*)' Transmit azimuth          : ',azim0
cf      write(  6,*)' (ODE) printout step  [km] : ',dels0
cf      write(  6,*)' Target range         [km] : ',sfin0
cf      write(  6,*)' '
cf      luo = luo + 1
cf      write(luo,501)
cf      write(luo,*)' NEW'
cf      write(luo,*)' Transm. from lat,lon [deg]: ',flat0,flon0
cf      write(luo,*)' Transmit azimuth          : ',azim0
cf      write(luo,*)' (ODE) printout step  [km] : ',dels0
cf      write(luo,*)' Target range         [km] : ',sfin0
cf      write(luo,*)' '

      abserr=abserr0
      relerr=relerr0
c     ASCII OUTPUT WILL BE ON fort.luo, starting with fort.11
cf      write(luo,508)

c     Integration is based on path-length /s
      dels     = dels0
      sfinal   = sfin0

c     IC => initial conditions on lat(1),lon(2), and azimuth(3)
      uIC(1)    =  flat0*torad
      uIC(2)    =  flon0*torad
      uIC(3)    =  azim0*torad
      u(1)      =  uIC(1)
      u(2)      =  uIC(2)
      u(3)      =  uIC(3)

c     INITIALIZE O.D.E. SOLVER AT sout = s = s0 = 0.
      s         =  s0
      sout      =  s
      iflag     = +1
c     ************************************************************
      call rkf45(ffr,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      nst       =  1
      if (iflag.gt.2) then
cf         PRINT*, ' WARNING: after 1-st step: iflag= ',iflag
cf         PRINT*,'relerr,abserr = ',relerr,abserr
         write (0,*) ' WARNING: after 1-st step: iflag= ',iflag
         write (0,*) 'relerr,abserr = ',relerr,abserr
        DO 205 i=1,21
        WRITE (0,*) ' i,work : ',i,work(i)
  205   CONTINUE
        DO 210 i=1,5
        WRITE (0,*) ' i,iwork: ',i,iwork(i)
  210   CONTINUE
        WRITE (0,*) ' RE-SET to +2'
        iflag   = +2
      endif
cf      write(luo,510) s,(u(i)/torad,i=1,neqn)
      if (startr .eq. 0.d0) then
cf         write(6,1510) s * 1000,(u(i)/torad,i=1,neqn)
         outlong = (((((u(2)/torad)/360)+10) - 
     *        int((((u(2)/torad)/360)+10))) * 360)
         write(6,1510) s * 1000,u(1)/torad,outlong,u(3)/torad
      endif
      if (sout1.gt.0.) then
        sout      = sout1
c       ************************************************************
        call rkf45(ffr,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c       ************************************************************
cf        write(  6,510) s,(u(i)/torad,i=1,neqn),iflag
cf        write(luo,510) s,(u(i)/torad,i=1,neqn)
cf        write(6,1510) s * 1000,(u(i)/torad,i=1,neqn)
        outlong = (((((u(2)/torad)/360)+10) - 
     *       int((((u(2)/torad)/360)+10))) * 360)
        write(6,1510) s * 1000,u(1)/torad,outlong,u(3)/torad
        if (iflag.gt.2)  goto 230
      endif

c     ---------------------
  220 sout      = s + dels
c     ---------------------
      nst = nst + 1
      if (sout.gt.sfinal) then
        sout    = sfinal
      endif

c     it is also assumed that the iflag = +2 => normal integration
c     ************************************************************
      call rkf45(ffr,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      if (iflag.gt.2) then
cf        PRINT*,' WARNING: after step# ',nst,' : iflag= ',iflag
        write (0,*)' WARNING: after step# ',nst,' : iflag= ',iflag
        iflin  = iflag
c       *************************************************
        call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c       *************************************************
cf        if (iflout.eq.9)  goto 240
        iflag  = iflout
      else
        relerr = relerr0
      endif

cf      write(luo,510) s,(u(i)/torad,i=1,neqn)
cf      write(6,1510) s * 1000,(u(i)/torad,i=1,neqn)
      outlong = (((((u(2)/torad)/360)+10) - 
     *     int((((u(2)/torad)/360)+10))) * 360)
      write(6,1510) s * 1000,u(1)/torad,outlong,u(3)/torad

      if (s.le.sfinal) then
        if (iflag.gt.2)  goto 230
        if (s.lt.sfinal) goto 220
      endif
      if (s.ge.sfinal) then
cf        write( 6,510) s,(u(i)/torad,i=1,neqn),iflag
c       endrun diagnostic :
cf        write(luo,555) s,(u(i)/torad,i=1,neqn)
        goto 240
      endif

  230 continue
cf    print 501
c     branching to diagnostics of abnormal performance :
      iflin = iflag
c     *************************************************
      call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c     *************************************************
      if (iflout.eq.9)  goto 240
      iflag = iflout
      goto 220

  240 continue
c-      CLOSE(luo)
cf      goto 100

  250 continue
cf      print*, ' '
cf      print*, ' '
cf      if (luo.eq.10) then
cf        print*,' WARNING: no output, because there is no MENU !'
cf      else
cf        luz = luo - 10
cf        print*,' RUN COMPLETED :'
cf        print*,' there is output in ',luz,' generic (fort...) files'
cf        print*,' last output is in file : fort.',luo
cf      endif
cf      CLOSE(7)
      goto 999

  950 write (0,*) ' early EOF on SOFAR data base'


  999 end











