      SUBROUTINE rgGEODES(Y1,X1,Y2,X2,S,NPTS,a12,a21,azis,R0,X,Y,IDB)
c=====================================================================
c# this subroutine finds the spherical coordinates along a geodesic between 
c# points at (note y comes first, as in lat, long) (Y1,X1) and (Y2,X2) 
c# for the (meter) arc ranges given in vector S() of length NPTS
c# it is most often used for finding the true horizontal locations of
c# ray paths between source and receiver.

C INPUT LATs AND LONGs IN DECIMAL DEGREES. OUTPUT DIST IN M., AND
C AZIMUTHS IN RADIANS positive clockwise (like on the compass) WITH NORTH = 0.
c A12 = AZIMUTH FROM 1 TO 2, A21 = AZIMUTH FROM 2 TO 1
c     (a21 now points forward, not back to the origin)
c R0 is the length in meters, and X(I),Y(I) are the positions of R(i)
C IDB = DEBUGGING PARAMETER
c=====================================================================c
c     this routine is a newer version of geodes.  it has been significantly
c     altered, in that the cartesian coordinate feature has been removed 
c     and the spherical coordinates calculations have been improved using 
c     subroutine versions of George Dworski's iGEOD.f and ddr18.f, called 
c     gdist.f and sgshoot.f.

c     where possible, Bruce Cornuelle's conventions have been kept, namely,
c     keeping the IDB variable, but (obviously) the parameter list is 
c     different.  x1,y1,x2,y2,s,npts,a12,a21,r0,x,y, and idb work the
c     same but igeod, rlat and rlong have been omitted, and azis, an array 
c     that stores the azimuth angles for each point, has been added.

c     craig friesen 6/30/92

cf  CHANGE: oct 93, altered to handle refracted geodesics

c=====================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      real*8 S(*),X(*),Y(*),azis(*)
      real*8 tranges(3),tlats(3),tlongs(3),tazis(3)
      integer i,tnum
      pi = 4.d0*atan(1.d0)
      CANGLE = 2.D0*PI/360.D0


      IF (IDB .GT. 2) WRITE (6,1001) 
     *     Y1,X1,Y2,X2,S(1),NPTS,A12,A21,azis(1),R0,X(1),Y(1),IDB
1001  FORMAT (' GEODES CALLING PARAMETERS:'
     * ,/,4f13.6,/,f13.1,I13,3f13.6,/,f13.1,2f13.6,I13)

 1    format (100f14.4)

c     compute the main geodesic
      call rgdist (y1,x1,y2,x2,a12,a21,r0)


c     find each point, (lat, long and azimuth)
      do 111, i = 1,npts
         trange = s(i)
         tinc = trange

c     convert to radians for srgshoot
         a12 = a12 / cangle
         call srgshoot   (x1,y1,a12,tinc,trange,
     *        tranges,tlats,tlongs,tazis)
c     convert back
         a12 = a12 * cangle

c         tranges(tnum)
         tnum = 2
         x(i) = tlongs(tnum)
         y(i) = tlats(tnum)
         azis(i) = tazis(tnum)


 111  continue

      if (a12 .ge. pi) a12 = a12 - 2*pi
      if (a21 .ge.2*pi) a21 = a21 - 2*pi
   

      RETURN
      END
