      program iFERM

Cf  =  Craig Friesen's comments

Cf    CHANGE: 4/2/93 to non-interactive, command line parameter
Cf    passing version.  Interactive portions suppressed.
Cf    output of parts 1 and 2 (great circle and non-refraced
Cf    geodesic) and all but final output of part 3 (FERMAT path)
Cf    have been suppressed.

c     DOUB.PREC.: iterative horiz.refrac. FERMAT path "A"-to-"B"
c     System of 2+1 GEODESIC O.D.E. (for lat, long, and azimuth)
c     driver for the /Drkf45/,/Drkfs/, and /Dfehl/ integr. group,
c     ellipsoidal earth GEODESIC version using /Dfff,/Dffr,/DgeordG,
c     and the SOFAR sound speed data matrix ~/LEVITyear/axisvel.

c     prog/iFERM   status :        18 March 1993

c     J. G. Dworski

c     H a n d l i n g
c     ===============

c     Support reqr'd : /Drkf45,/Drkfs,/Dfehl,/Dfff,/Dffr,/DgeordG
c     ( for initialization :  subr.: /sphetr.f and /angfra.f )

c     Initialization by extracting code of our /iGRCL.f :
c     "great circle" between STA"A" and STA"B", with interactive I/O
c     and then solve for geodesic on smooth WGS84 ellipsoid

c     Interactive input :
c     latitude and longitude [deg] of Stations A and B.
c        To operate with 1 [m] range resolution, these angles must be
c        specified to a precision of 1.e-5 [deg], so that - in general -
c        the angles should have 7 to 8 significant digits.
c        Result (screen, hard-copy, and feed-in to iteration) :
c        (a) the A-to-B azimuth [deg] at A,
c        (b) the A-to-B azimuth [deg] at B,
c        (c) the distance [km] between A and B along the great circle,

c     Initialization part :
c     dlaA    = latitude  [deg] at Sta A
c     dloA    = longitude [deg] at Sta A
c     dlaB    = latitude  [deg] at Sta B
c     dloB    = longitude [deg] at Sta B
c     ss      = great circle distance [km] between A and B
c     azA     = A-to-B azimuth [deg] at A

c     Iterative loop on geodesics or FERMAT path :
c     s       = distance  [km]
c     y(1)    = latitude  [rad]
c     y(2)    = longitude [rad]
c     y(3)    = azimuth   [rad] (positive cwse from North)
c     yp(i)   = (first) time derivatives of /y/ above, i=1,2,3
c       remark... this program must provide
c                  (in addition to error and output management) :
c               (1) initial conditions on vector /y/
c               (2) initial distance in /s/ (typically 0.)

c     ==========================================================

      implicit double precision (a-h,o-z)

      external fff,ffr

      double precision r0,pi,torad,dloBA,aloBA,depsg,depsf
      double precision dlaA,dloA,dlaB,dloB
cf      double precision zdeg,zmin,zsec
      double precision daxc,daxs,dbxc,dbxs,dccxc,dccxs,dcxc,dcxs
      double precision cosaa,sinaa,cosbb,sinbb
      double precision da,db,dc,daa,dbb,dcc
c      double precision dbcoA,dacoB,Eps,Csid,Aang,Bang
      parameter( pi=3.141592653589d0,pi2=2.d0*pi )
      parameter( torad=1.745329252d-02 )
c     radius, r0[km], of equal AREA sphere (to WGS 84 ellipsoid) :
      parameter( r0=6.371005d+03 )
cf      parameter( luo=11 )
cf      logical       yes,minsec,echo
      logical       echoatend

cf      parameter( s0=0.d0,sout1=0.d0,dels=100.d0 )
      parameter( s0=0.d0,sout1=0.d0,dels=10.d0 )
      parameter( neqn=3,imax=180,jmax=360 )
      double precision  glat,cosl,rpve,dsina,ddummy
      double precision  dpi
      double precision  s,uIC(neqn),u(neqn),sout,relerr,abserr
      double precision  work(21)
      integer           iwork(5),iflag
c      character*72     outfil
      character*100     FILE
      real              zmat(imax,jmax)

      common/same/      i1old,j1old
      common/datmat/    zmat
      data     depsg/5.d-4/,depsf/5d-3/

      character*100 strTemp

  501 format(/)
  505 format(a)
  506 format(1x,a)
  507 format(/'  FINAL geodesic :   Dtang [km] Dnorm',t43,'#iterates:')
  508 format('        RANGE      LAT        LON        AZI'/
     1       '        [km]      [deg]      [deg]      [deg]')
  509 format(' Geodesic and Fermat path iter. :'/' "A" ->'/' "B" ->')
  510 format(f12.3,3f12.6,i5)
 1510 format(f12.3,3f12.6,i5)
  511 format(2f12.3)
  512 format(15x,2f11.3,i11)
  514 format(' --------------------------------------------------')
  516 format(' Lat,Lon for Sta"A":',2f15.8,' [deg]')
  517 format(' Lat,Lon for Sta"B":',2f15.8,' [deg]')
  527 format(/'  FINAL Fermat path :   Dtang [km] Dnorm',t43,'#iter :')
  528 format('        RANGE      LAT        LON        AZI'/
     1       '        [km]      [deg]      [deg]      [deg]')
  529 format(' Hor.Fermat path iteration :'/' "A" ->'/' "B" ->')
  547 format(' (A -> B) AZIMUTH  angle  at staA :' ,f15.8,' [deg]')
  548 format(' (A -> B) AZIMUTH  angle  at staB :' ,f15.8,' [deg]')
  551 format(' GREAT CIRC. distance |staA-staB| :' ,f15.4,' [ km]')
  555 format(f12.3,4f12.6)
  560 format(i5,2f11.4,f10.2,1Pe14.4)

c     start of executables :
c     ---------------------

cf    get and check arguments off command-line:

      m = iargc()
      if (m .ne. 4) then
         write (0,*)'wrong number of arguments.'
         write (0,*)'Usage: rgrange lat(origin) long(origin) '
         write (0,*)'               lat(destination) long(destination)'
         write (0,*)
         STOP
      endif

      call getarg(1, strTemp)
      read (strTemp, *) dlaA
      if (abs(dlaA).gt.90.d0) then
         write (0,*)' ERROR : latitude ',dlaA,' is out of bounds'
         STOP
      endif

      call getarg(2, strTemp)
      read (strTemp, *) dloA
      if (dloA.lt.-180.d0 .or. dloA.gt.360.d0) then
         write (0,*)'ERROR : longitude ',dloA,' is out of bounds'
         STOP
      endif
      
      call getarg(3, strTemp)
      read (strTemp, *) dlaB
      if (abs(dlaB).gt.90.d0) then
         write (0,*)' ERROR : latitude ',dlaB,' is out of bounds'
         STOP
      endif

      call getarg(4, strTemp)
      read (strTemp, *) dloB
      if (dloB.lt.-180.d0 .or. dloB.gt.360.d0) then
        write (0,*)' ERROR : longitude ',dloB,' is out of bounds'
        STOP
      endif
      print *
      print *,'source lat       [deg]:',dlaA
      print *,'source long      [deg]:',dloA
      print *,'receiver lat     [deg]:',dlaB
      print *,'receiver long    [deg]:',dloB
      print *,' '
      
cf      print 501
cf      print*,' Search for horiz.refrac. FERMAT path has started'

cf      print*,' WGS84 (earth ellipsoid) + SOFAR data > FERM.dat'
cf      print 501
cf     they can pipe it themselves if they want it.

c     NOTE reduced printout schedule :
cf      dels0 = 1000.d0
      dels0 = 10.d0

c     INITIALIZATION : by extracting code of our /iGRCL.f :
c     "great circle" between STA"A" and STA"B", with interactive I/O


c     SOUND SPEED FIELD :

c     input: minimum sound speed file "axisvel" (from ~/LEVITyear) must
c     have been copied into the working directory  :

cf     Let's assume they have it

cf      call system(' ls -l axisvel ')
cf      print*,' "axisvel" must be present ;   continue ?  [CR=yes]'
cf      if (.not.yes(1)) then
cf        goto 999
cf      endif

cf      print*,' '
c     (180 words/record, 360 records)
      len1 = imax*4
cUSER the following line must contain the correct path for the 
cUSER refracted geodesics package:
      CALL GETENV('ATOC_LIBRARY', FILE)
      OPEN(3,file=FILE(1:lnblnk(FILE))//'/axisvel',form='unformatted',
     1     access='direct',recl=len1)
      do 050 jm = 1,jmax
c       ---------------------------------------------
        read(3,rec=jm) (zmat(i,jm),i=1,imax)
c        read(3,rec=jm,END=950) (zmat(i,jm),i=1,imax)
c       ---------------------------------------------
  050 continue
cf      print*,' SOFAR axis sound speed data reading completed'
cf      print*,' '
      CLOSE(3)

cf      OPEN(luo,file='FERM.dat')

cf  055 print*,' input co-ord. in [deg] (including fraction) ? [CR=yes]'
cf      print*,'        ( alternative is [deg],[min],[sec] )'
cf      if (.not.yes(1)) then
cf        minsec = .true.
cf        print*,' when asked, configure input as 3 numbers'
cf        print*,' separated by blanks, for [deg] [min] [sec]'
cf      else
cf        minsec = .false.
cf        print*,' when asked, configure input as 1 number, incl. fract.'
cf      endif

cf  060 print*,' ENTER (transmitter : sta A) LATITUDE [deg] :'
cf        if (minsec) then
cf          print*,' (3 numbers for : [deg], [min], [sec])'
cf          read*, zdeg,zmin,zsec
c         ************************************
cf          call angfra(dlaA,.false.,zdeg,zmin,zsec)
c         ************************************
cf        else
cf          read*, dlaA
cf        endif
        if (abs(dlaA).gt.90.d0) then
           write (0,*) ' ERROR : latitude ',dlaA,' is out of bounds'
cf          goto 060
          goto 999
        endif

cf  065 print*,' ENTER (transmitter : sta A) LONGITUDE [deg] :'
cf        if (minsec) then
cf          print*,' (3 numbers for : [deg], [min], [sec])'
cf          read*, zdeg,zmin,zsec
cf          call angfra(dloA,.false.,zdeg,zmin,zsec)
cf        else
cf          read*, dloA
cf        endif
        if (dloA.lt.-180.d0 .or. dloA.gt.360.d0) then
          write (0,*) ' ERROR : longitude ',dloA,' is out of bounds'
cf          goto 065
          goto 999
        endif

cf      print 501
cf  070 print*,' ENTER (receiver : sta B) LATITUDE [deg] :'
cf      if (minsec) then
cf        print*,' (3 numbers for : [deg], [min], [sec])'
cf        read*, zdeg,zmin,zsec
c       ******************************************
cf        call angfra(dlaB,.false.,zdeg,zmin,zsec)
c       ******************************************
cf      else
cf        read*, dlaB
cf      endif
      if (abs(dlaB).gt.90.d0) then
        write (0,*) ' ERROR : latitude ',dlaB,' is out of bounds'
cf        goto 070
        goto 999
      endif

cf  075 print*,' ENTER (receiver : sta B) LONGITUDE [deg] :'
cf      if (minsec) then
cf        print*,' (3 numbers for : [deg], [min], [sec])'
cf        read*, zdeg,zmin,zsec
cf        call angfra(dloB,.false.,zdeg,zmin,zsec)
cf      else
cf        read*, dloB
cf      endif
      if (dloB.lt.-180.d0 .or. dloB.gt.360.d0) then
         write (0,*) ' ERROR : longitude ',dloB,' is out of bounds'
cf        goto 075
        goto 999
      endif

cf      print 501
cf      write(  6,516) dlaA,dloA
cf      write(  6,517) dlaB,dloB
cf      print*,' O.K. ?    [CR=yes]'
cf      if (.not.yes(1)) goto 055
cf      print*,' silence echo ?   CR=yes]'
cf      if (yes(1)) then
cf        echo = .false.
cf      else
        echoatend = .false.
cf        write(  6,*  ) ' (ODE) printout step  [km] : ',dels0
cf      endif

cf      write(  6,509)
cf      write(  6,514)
cf      write(  6,516) dlaA,dloA
cf      write(  6,517) dlaB,dloB
cf     write(luo,509)
cf     write(luo,514)
cf     write(luo,516) dlaA,dloA
cf     write(luo,517) dlaB,dloB

c     make WESTERN HEMISPHERE longitudes to be NEGATIVE
      if (dloA.gt.180.) dloA=dloA-360.d0
      if (dloB.gt.180.) dloB=dloB-360.d0

c     compute  CO-LATITUDES
      da    =  (90.d0 - dlaB)  * torad
      db    =  (90.d0 - dlaA)  * torad
c     and compute LONGITUDE DIFFERENCE
      dloBA =   dloB  - dloA

c     restrict longitude difference to interval +/- 180 degrees
      if     (dloBA.gt. 180.d0) then
        dloBA = dloBA - 360.d0
      elseif (dloBA.lt.-180.d0) then
        dloBA = dloBA + 360.d0
      endif
c     and then consider only the absolute value of the polar angle
      aloBA   = abs(dloBA)


c     P A R T   1  :
c     ==============

c     A closed form "starter" for the iterative search
c     is the computed "great circle path", ss, and its
c     azimuth at station A, azA .

c     c o m p u t e   from first principles :
c     oblique spherical triangle :
c     ----------------------------
      dcc   =  dabs(dloBA) * torad
      daxc  =  dcos(da)
      daxs  =  dsin(da)
      dbxc  =  dcos(db)
      dbxs  =  dsin(db)
      dccxc =  dcos(dcc)
      dccxs =  dsin(dcc)
      dcxc  =  daxc*dbxc + daxs*dbxs*dccxc
      if (dcxc.le.-1.d0) then
        dc  =  pi
      else
        dc  =  dacos(dcxc)
      endif
      dcxs  =  dsin(dc)
      if (dcxs.lt.1.d-8) then
        cosaa = 1.d0
        sinaa = 0.d0
        cosbb = 1.d0
        sinbb = 0.d0
      else
        cosaa = (daxc-dbxc*dcxc)/(dbxs*dcxs)
        sinaa =  dccxs/dcxs*daxs
        cosbb = (dbxc-daxc*dcxc)/(daxs*dcxs)
        sinbb =  dccxs/dcxs*dbxs
      endif
      daa   =  datan2(sinaa,cosaa)
      dbb   =  datan2(sinbb,cosbb)
c     cludge :
      daaanom = 180.d0-abs(daa)/torad
      dbbanom = 180.d0-abs(dbb)/torad
      if (daaanom.lt.0.02d0) daa=abs(daa)
      if (dbbanom.lt.0.02d0) dbb=abs(dbb)

      cgc   =  real(dc/torad)
      ss    =  real(dc*r0)
c     ----------------------------

      if (dloBA.ge.0.d0) then
        azA =  real(         daa /torad)
        azB =  real((2.d0*pi-dbb)/torad)
      else
        azA =  real((2.d0*pi-daa)/torad)
        azB =  real((        dbb)/torad)
      endif
      if (azB.ge.180.d0) then
        azB = azB-180.d0
      else
        azB = azB+180.d0
      endif

cf      write(  6,547) azA
cf      write(  6,548) azB
cf      write(  6,  *)' end of PART 1  (great circle on sphere)'
cf      write(  6,551) ss
cf      write(  6,514)
cf     write(luo,547) azA
cf     write(luo,548) azB
cf     write(luo,  *)' end of PART 1  (great circle on sphere)'
cf     write(luo,551) ss
cf     write(luo,514)

      dpi    =dble(pi)
      abserr0=1.5d-9
      relerr0=1.0d-9

cf      if (echo) then
cf        write(  6,*) ' by default ... '
cf        write(  6,*) ' absolute error = ',abserr0
cf  080   write(  6,*) ' relative error = ',relerr0
cf        print*,' O.K.?  [CR=yes]'
cf        if (.not.yes(1)) then
cf          print*,' ENTER new relative error :'
cf          print*,' (we do not change the absolute error)'
cf          read*, relerr0
cf          goto 080
cf        endif
cf      endif

c     P A R T   2  :
c     ==============

c     Iterative search for the "geodesic path" begins with
c     the just completed "great circle path" as starter :

      niter  = -1
      azim0  = azA
      if (abs(azim0-180.d0).lt.0.01d0 .and. ss.gt.2.d4) then
        write(0,*) ' SORRY, geodesic heading across S.Pole'
cf        write(6,*) '        apply singular analysis'
        goto 950
      elseif (abs(azim0)   .lt.0.01d0 .and. ss.gt.2.d4) then
        write(0,*) ' SORRY, geodesic heading across N.Pole'
cf        write(6,*) '        apply singular analysis'
        goto 950
      endif

c     targeted range, sfin0, is the great circle distance, ss
      sfin0  = ss
c     REMARK: integration is based on path-length /s
      sfinal = sfin0
      chzold = 0.d0
c-      disold = 0.d0
      nitlim = 100
      nsam   = 0

c     ITERATIVE SEARCH LOOP :
  100 continue
      niter = niter + 1
cf      if (echo) then
cf        write(  6,*) ' '
cf        write(  6,*) ' ITERATE # ',niter
cf        write(  6,*) ' Transm. from lat,lon [deg]: ',dlaA,dloA
cf        write(  6,*) ' Transmit azimuth     [deg]: ',azim0
cf        write(  6,*) ' Targeted range       [km] : ',sfinal
cf        write(  6,508)
cf       write(luo,*) ' '
cf       write(luo,*) ' ITERATE # ',niter
cf       write(luo,*) ' Transm. from lat,lon [deg]: ',dlaA,dloA
cf       write(luo,*) ' Transmit azimuth     [deg]: ',azim0
cf       write(luo,*) ' Targeted range       [km] : ',sfinal
cf       write(luo,508)
cf      endif

      abserr=abserr0
      relerr=relerr0

c     IC => initial conditions [rad] : lat(A),lon(A), and azimuth(at A)
c                 ( reference: [rad] : lat(B),lon(B))
      dlaAr     =  dlaA *torad
      dloAr     =  dloA *torad
      dlaBr     =  dlaB *torad
      dloBr     =  dloB *torad
      uIC(1)    =  dlaAr
      uIC(2)    =  dloAr
      uIC(3)    =  azim0*torad
      u(1)      =  uIC(1)
      u(2)      =  uIC(2)
      u(3)      =  uIC(3)

      glat      =  dble(uIC(1))
      dsina     =  dble(sin(uIC(3)))
c     *******************************************
      call geordG( glat,ddummy,cosl,ddummy,rpve )
c     *******************************************

c     INITIALIZE O.D.E. SOLVER AT sout = s = s0 = 0.
      s         =  s0
      sout      =  s
      iflag     = +1
c     ************************************************************
      call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      nst       =  1
      if (iflag.gt.2) then
        write (0,*) ' WARNING: after 1-st step: iflag= ',iflag
        write (0,*) ' relerr,abserr = ',relerr,abserr
        DO 205 i=1,21
        write (0,*) ' i,work : ',i,work(i)
  205   CONTINUE
        DO 210 i=1,5
        write (0,*) ' i,iwork: ',i,iwork(i)
  210   CONTINUE
        write (0,*) ' RE-SET to +2'
        iflag   = +2
      endif
cf      if (echo) then
cf       write(luo,510) s,(u(i)/torad,i=1,neqn)
cf      endif

      if (sout1.gt.0.) then
        sout      = sout1
c       ************************************************************
        call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c       ************************************************************
cf        if (echo) then
cf          write(  6,510) s,(u(i)/torad,i=1,neqn),iflag
cf         write(luo,510) s,(u(i)/torad,i=1,neqn)
cf        endif
        if (iflag.gt.2)  goto 230
      endif

c     ---------------------
  220 sout      = s + dels
c     ---------------------
      if (sout.gt.sfinal) then
        sout    = sfinal
      endif
      nst = nst + 1

c     it is also assumed that the iflag = +2 => normal integration
c     ************************************************************
      call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      if (iflag.gt.2) then
        iflin  = iflag
c       *************************************************
        call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c       *************************************************
        if (iflout.eq.9)  goto 240
        iflag  = iflout
      else
        relerr = relerr0
      endif

cf      if (echo) then
cf        if (mod(s,dels0).eq.0.d0) then
cf          write(luo,510) s,(u(i)/torad,i=1,neqn)
cf          write(6,510) s,(u(i)/torad,i=1,neqn)
cf        endif
cf      endif

      if (s.le.sfinal) then
        if (iflag.gt.2)  goto 230
        if (s.lt.sfinal) goto 220
      endif

c     THE n-th ITERATE HAS BEEN COMPUTED :
      if (s.ge.sfinal) then
cf        if (echo) then
cf          write(  6,510) s,(u(i)/torad,i=1,neqn),iflag
cf          write(luo,510) s,(u(i)/torad,i=1,neqn),iflag
cf        endif

c       EVALUATE NEED FOR ADDITIONAL ITERATES :
c      (a) mark terminal point as origin of local scheme :
c                ************
         ufx1  = xprinc(u(2))
c                ************
         ufy1  = u(1)
         cosy1 = cos(ufy1)
         ufazE = u(3)
         azimE = ufazE/torad
c      (b) express "receiver" point"B" relative to origin :
         x0    = dloBr - ufx1
         if (abs(x0).gt.0.2d0) then
           x0  = x0 - pi2*sign(1.d0,x0)
         endif
         y0    = dlaBr - ufy1
c        expressed in [km] (with terminal point as (0,0)) :
         x0    = r0*x0*cosy1
         y0    = r0*y0
cf         if (echo) then
cf           write(  6,*  ) ' (rec rel term)        : Dx,Dy [km] :'
cf           write(  6,511) x0,y0
cf          write(luo,*  ) ' (rec rel term)        : Dx,Dy [km] :'
cf          write(luo,511) x0,y0
cf         endif
c      (c) def: unit"tangent"vector (1km long) to the geodesic at (0,0) :
c        convert azimuth into "mathematical ccwse" angles (0,+/-pi)
c                *********************
         alphE = xprinc(pi/2.d0-ufazE)
c                *********************
         x2    =   cos(alphE)
         y2    =   sin(alphE)
c      (d) inner product: X0dotX2 : projection onto tangent:
         proj  =   x2*x0 + y2*y0
c      (e) outer product: X0crossX2 : distance from tangent:
         dist  =   x0*y2 - x2*y0
         chazi = dist/sfinal
cf         if (echo) then
cf           write(  6,*  ) ' (proj,norm to tangent): Ds,Dn [km] :'
cf           write(  6,511) proj,dist
cf          write(luo,*  ) ' (proj,norm to tangent): Ds,Dn [km] :'
cf          write(luo,511) proj,dist
cf         endif

       if ((abs(proj).lt.depsg .and. abs(dist).lt.depsg)
     1  .or. niter.gt.nitlim) then
         goto 240
cf       else
cf         if (echo) then
cf           write(  6,*)' change transmit aximuth by [rad] = ',chazi
cf          write(luo,*)' change transmit aximuth by [rad] = ',chazi
cf         endif
       endif
        sfinal = sfinal + proj

        if (chzold*chazi.ge.0.d0) then
          nsam=nsam+1
        else
          nsam=0
        endif
        if     (nsam.le.3) then
          cfac=1.0d0
        elseif (nsam.eq.0) then
          cfac=0.8d0*cfac
        else
          cfac=1.2d0*cfac
        endif
        chazi  = cfac*chazi
        azim0  = azim0  + chazi/torad
        chzold = chazi

        goto 100
      endif

  230 print 501
c     branching to diagnostics of abnormal performance :
      iflin = iflag
c     *************************************************
      call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c     *************************************************
      if (iflout.eq.9)  goto 240
      iflag = iflout
      goto 220

  240 continue
        glat      =  dble(u(1))
c       *******************************************
        call geordG( glat,ddummy,cosl,ddummy,rpve )
c       *******************************************

cf      end iterate (on geodesic path) diagnostic :
cf        write(  6,*)' '
cf        write(  6,*)' end of PART 2  (spheroidal geodesic)'
cf        write(  6,*)' Convergence results :'
cf        write(  6,*)' Dtang[km] (along  path) : ',proj
cf        write(  6,*)' Dnorm[km] (across path) : ',dist
cf        write(  6,*)' precision[km] toler     : ',depsg
cf        write(  6,*)' iterates used #         : ',niter
cf       write(  6,507)
cf       write(  6,512) proj,dist,niter
cf       write(  6,508)
cf        write(luo,*)' '
cf        write(luo,*)' end of PART 2  (spheroidal geodesic)'
cf        write(luo,*)' Convergence END results :'
cf        write(luo,*)' Dtang[km] (along  path) : ',proj
cf        write(luo,*)' Dnorm[km] (across path) : ',dist
cf        write(luo,*)' precision[km] toler     : ',depsg
cf        write(luo,*)' iterates used #         : ',niter
cf       write(luo,507)
cf       write(luo,512) proj,dist,niter
cf       write(luo,508)
cf        write(  6,555) s0,dlaA,dloA,azim0
cf        write(  6,555) s,(u(i)/torad,i=1,neqn)
cf       write(luo,555) s0,dlaA,dloA,azim0
cf       write(luo,555) s,(u(i)/torad,i=1,neqn)

c     P A R T   3  :
c     ==============

c     Iterative search for "horizontal Fermat path" starts with
c     the just completed "geodesic path" as starter :

c     targeted range, ss, is no longer the great circle, but the just
c     computed ellipsoidal distance, s, and the "best" azimuth, azim0
      ss = s

      niter  = -1

      sfin0  = ss
c     REMARK: integration is based on path-length /s
      sfinal = sfin0
      chzold = 0.d0
c-      disold = 0.d0
      nitlim = 100
      nsam   = 0

c     ITERATIVE SEARCH LOOP :

cf    =======

  300 continue
      i1old = 1
      j1old = 1
      niter = niter + 1
cf      if (echo) then
cf        write(  6,*) ' '
cf        write(  6,*) ' ITERATE # ',niter
cf        write(  6,*) ' Transm. from lat,lon [deg]: ',dlaA,dloA
cf        write(  6,*) ' Transmit azimuth     [deg]: ',azim0
cf        write(  6,*) ' Targeted range       [km] : ',sfinal
cf        write(  6,508)
cf       write(luo,*) ' '
cf       write(luo,*) ' ITERATE # ',niter
cf       write(luo,*) ' Transm. from lat,lon [deg]: ',dlaA,dloA
cf       write(luo,*) ' Transmit azimuth     [deg]: ',azim0
cf       write(luo,*) ' Targeted range       [km] : ',sfinal
cf       write(luo,508)
cf      endif

      abserr=abserr0
      relerr=relerr0

c     IC => initial conditions [rad] : lat(A),lon(A), and azimuth(at A)
c                 ( reference: [rad] : lat(B),lon(B))
      dlaAr     =  dlaA *torad
      dloAr     =  dloA *torad
      dlaBr     =  dlaB *torad
      dloBr     =  dloB *torad
      uIC(1)    =  dlaAr
      uIC(2)    =  dloAr
      uIC(3)    =  azim0*torad
      u(1)      =  uIC(1)
      u(2)      =  uIC(2)
      u(3)      =  uIC(3)

      glat      =  dble(uIC(1))
      dsina     =  dble(sin(uIC(3)))
c     *******************************************
      call geordG( glat,ddummy,cosl,ddummy,rpve )
c     *******************************************

c     INITIALIZE O.D.E. SOLVER AT sout = s = s0 = 0.
      s         =  s0
      sout      =  s
      iflag     = +1
c     ************************************************************
      call rkf45(ffr,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      nst       =  1
      if (iflag.gt.2) then
        write (0,*) ' WARNING: after 1-st step: iflag= ',iflag
        write (0,*) ' relerr,abserr = ',relerr,abserr
        DO 405 i=1,21
        write (0,*) ' i,work : ',i,work(i)
  405   CONTINUE
        DO 410 i=1,5
        write (0,*) ' i,iwork: ',i,iwork(i)
  410   CONTINUE
        write (0,*) ' RE-SET to +2'
        iflag   = +2
      endif
cf      if (echo) then
cf       write(luo,510) s,(u(i)/torad,i=1,neqn)
cf      endif

      if (sout1.gt.0.) then
        sout      = sout1
c       ************************************************************
        call rkf45(ffr,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c       ************************************************************
cf        if (echo) then
cf          write(  6,510) s,(u(i)/torad,i=1,neqn),iflag
cf         write(luo,510) s,(u(i)/torad,i=1,neqn)
cf        endif
        if (iflag.gt.2)  goto 430
      endif

c     ---------------------
  420 sout      = s + dels
c     ---------------------
      if (sout.gt.sfinal) then
        sout    = sfinal
      endif
      nst = nst + 1

c     it is also assumed that the iflag = +2 => normal integration
c     ************************************************************
      call rkf45(ffr,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      if (iflag.gt.2) then
        iflin  = iflag
c       *************************************************
        call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c       *************************************************
        if (iflout.eq.9)  goto 440
        iflag  = iflout
      else
        relerr = relerr0
      endif

cf      if (echoatend) then
cf        if (mod(s,dels0).eq.0.d0) then
cf        if (mod(s,inc).eq.0.d0) then
cf         write(luo,510) s,(u(i)/torad,i=1,neqn)
cf          write(6,1510) s * 1000,(u(i)/torad,i=1,neqn)
cf        endif
cf      endif
      if (s.le.sfinal) then
        if (iflag.gt.2)  goto 430
        if (s.lt.sfinal) goto 420
      endif

c     THE n-th ITERATE HAS BEEN COMPUTED :
cf      if (s.ge.sfinal) then
cf         if (echoatend) then
cf            write(  6,510) s,(u(i)/torad,i=1,neqn),iflag
cf            write(luo,510) s,(u(i)/torad,i=1,neqn),iflag
cf         endif
cf      endif
cf    =====
        if (echoatend) then 
           go to 998
        endif


c       EVALUATE NEED FOR ADDITIONAL ITERATES :
c      (a) mark terminal point as origin of local scheme :
c                ************
         ufx1  = xprinc(u(2))
c                ************
         ufy1  = u(1)
         cosy1 = cos(ufy1)
         ufazE = u(3)
         azimE = ufazE/torad
c      (b) express "receiver" point"B" relative to origin :
         x0    = dloBr - ufx1
         if (abs(x0).gt.0.2d0) then
           x0  = x0 - pi2*sign(1.d0,x0)
         endif
         y0    = dlaBr - ufy1
c        expressed in [km] (with terminal point as (0,0)) :
         x0    = r0*x0*cosy1
         y0    = r0*y0
cf         if (echo) then
cf           write(  6,*  ) ' (rec rel term)        : Dx,Dy [km] :'
cf           write(  6,511) x0,y0
cf          write(luo,*  ) ' (rec rel term)        : Dx,Dy [km] :'
cf          write(luo,511) x0,y0
cf         endif
c      (c) def: unit"tangent"vector (1km long) to the geodesic at (0,0) :
c        convert azimuth into "mathematical ccwse" angles (0,+/-pi)
c                *********************
         alphE = xprinc(pi/2.d0-ufazE)
c                *********************
         x2    =   cos(alphE)
         y2    =   sin(alphE)
c      (d) inner product: X0dotX2 : projection onto tangent:
         proj  =   x2*x0 + y2*y0
c      (e) outer product: X0crossX2 : distance from tangent:
         dist  =   x0*y2 - x2*y0
         chazi = dist/sfinal
cf         if (echo) then
cf           write(  6,*  ) ' (proj,norm to tangent): Ds,Dn [km] :'
cf           write(  6,511) proj,dist
cf          write(luo,*  ) ' (proj,norm to tangent): Ds,Dn [km] :'
cf          write(luo,511) proj,dist
cf         endif

       if ((abs(proj).lt.depsf .and. abs(dist).lt.depsf)
     1  .or. niter.gt.nitlim) then
         goto 440
cf       else
cf         if (echo) then
cf           write(  6,*)' change transmit aximuth by [rad] = ',chazi
cf          write(luo,*)' change transmit aximuth by [rad] = ',chazi
cf         endif
       endif
        sfinal = sfinal + proj

        if (chzold*chazi.ge.0.d0) then
          nsam=nsam+1
        else
          nsam=0
        endif
        if     (nsam.le.3) then
          cfac=1.0d0
        elseif (nsam.eq.0) then
          cfac=0.8d0*cfac
        else
          cfac=1.2d0*cfac
        endif
        chazi  = cfac*chazi
        azim0  = azim0  + chazi/torad
        chzold = chazi

        if (niter.gt.10) then
          write (0,*) ' WARNING: (refractive) comp. passed 10 iterates'
cf          print*,' double the tolerance ? [CR=yes]'
cf          if (yes(1)) then
          write (0,*) ' doubling the tolerance'
            depsf = 2.d0*depsf
            niter = 0
cf          else
cf            print*,' WARNING: run ends now !'
cf            goto 440
cf          endif
        endif
        goto 300

  430 print 501
c     branching to diagnostics of abnormal performance :
      iflin = iflag
c     *************************************************
      call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c     *************************************************
      if (iflout.eq.9)  goto 440
      iflag = iflout
      goto 420

  440 continue
        glat      =  dble(u(1))
c       *******************************************
        call geordG( glat,ddummy,cosl,ddummy,rpve )
c       *******************************************

c       endrun diagnostic :
cf         write(  6,*)' '
cf         write(  6,*)' end of PART 3  (horizontal Fermat path)'
cf         write(  6,*)' Convergence results :'
cf         write(  6,*)' Dtang[km] (along  path) : ',proj
cf         write(  6,*)' Dnorm[km] (across path) : ',dist
cf         write(  6,*)' precision[km] toler     : ',depsf
cf         write(  6,*)' iterates used #         : ',niter
cf        write(  6,527)
cf        write(  6,512) proj,dist,niter
cf        write(  6,528)
cf==========
      niter = niter -1
      echoatend = .true.
cf      write(  6,1510) s0,dlaA,dloA,azim0
cf      pi=3.141592653589d0
      r2d = 360.d0 / (2.d0*pi)
      range = s * 1000
      print*,'range              [m]:',range
      print*,'source azimuthh  [deg]:',azim0
cf      print*,'receiver azimuth [deg]:',azir*r2d
      print*,'receiver azimuth [deg]:',(u(i)/torad,i=3,neqn)
      print*
      goto 300

 998  continue
c     THE n-th ITERATE HAS BEEN COMPUTED :
cf      if (s.ge.sfinal) then
cf        if (echoatend) then
cf          write(  6,510) s,(u(i)/torad,i=1,neqn),iflag
cf          write(  6,1510) s * 1000,(u(i)/torad,i=1,neqn)
cf         write(luo,510) s,(u(i)/torad,i=1,neqn),iflag
cf        endif
cf        endif
cf==========
cf        write(luo,*)' '
cf        write(luo,*)' end of PART 3  (horizontal Fermat path)'
cf        write(luo,*)' Convergence results :'
cf        write(luo,*)' Dtang[km] (along  path) : ',proj
cf        write(luo,*)' Dnorm[km] (across path) : ',dist
cf        write(luo,*)' precision[km] toler     : ',depsf
cf        write(luo,*)' iterates used #         : ',niter
cf       write(luo,527)
cf       write(luo,512) proj,dist,niter
cf       write(luo,528)
cf        last ones:
cf        write(  6,555) s0,dlaA,dloA,azim0
cf        write(  6,555) s,(u(i)/torad,i=1,neqn)
cf       write(luo,555) s0,dlaA,dloA,azim0
cf       write(luo,555) s,(u(i)/torad,i=1,neqn)
Cf     print*,' '
Cf     print*,' Hard-copy in FERM.dat'
Cf     print*,' '
Cf     CLOSE(luo)
      goto 999

  950 write (0,*) ' '
      write (0,*) ' WARNING: problems with input,'
      write (0,*) ' run cannot continue'

  999 stop
      end

