
      subroutine sgshoot (flon0,flat0,azim0,dels0,sfin0,
     *     ranges,lats,longs,azis)

c     this routine is a version of George Dworski's ddr18.f program
c     that calculates points along geodesics.

c     input:a lat and a long, an angle to shoot a geodesic on, an
c     increment (distance between points), and a total range. 

c     output: the ranges, lats, longs and geodesic azimuths of the points
c     along the geodesic.

c     This routine takes the (input) parameters from the subroutine call
c     rather than interactively.  Many of the options available on the
c     interactive version (ddr18.f), such as verifying the absolute and
c     relative errors, setting special ranges for the first output, etc.
c     have been eliminated.  some of the comment lines have been omitted,
c     too, so you may need to consult the origional ddr18.f program.
c                                                 craig friesen 7/92
c===================================================================
c       s       = distpance  [km]
c       y(1)    = latitude  [rad]
c       y(2)    = longitude [rad]
c       y(3)    = azimuth   [rad] (positive cwse from North)

c       yp(i)   = (first) time derivatives of /y/ above, i=1,2,3

c     this routine is now set up to return values in meters
c==========================================================
      implicit real*8 (a-h,o-z)
      parameter( neqn=3 )
      parameter( torad=1.745329252d-02 )
      parameter( pi=3.141592653589d0,pi2=2.d0*pi )
      parameter( r0=6.371005d+03,s0=0.d0)

      real*8  fk0,glat,cosl,rpve,dsina,ddummy
      real*8  azdc,cosc,dpi
      real*8  s,uIC(neqn),u(neqn),sout,relerr,abserr
      real*8  work(21)
      integer  iwork(5),iflag,count
      character*6  lead,blank
      logical     usec

      real*8 ranges(*),lats(*),longs(*),azis(*)

      common/Clairaut/  fk0,usec
c     (the Clairaut invariant fk0 is common to fff.f)
      external fff


      data     lead/'  GEOD'/,blank/'     '/

  510 format(4f12.6)

      if (sfin0 .lt. 5.d-2) then
         return
      endif

c     convert to km for calculations

      dels0 = dels0/1000.d0
      sfin0 = sfin0/1000.d0
      

      dpi    =dble(pi)
      abserr0=1.5d-9
      relerr0=1.0d-9

      sout1 = 0.d0


      abserr=abserr0
      relerr=relerr0

c     Integration is based on path-length /s
      dels     = dels0
      sfinal   = sfin0

c     IC => initial conditions on lat(1),lon(2), and azimuth(3)
      uIC(1)    =  flat0*torad
      uIC(2)    =  flon0*torad
      uIC(3)    =  azim0*torad
      u(1)      =  uIC(1)
      u(2)      =  uIC(2)
      u(3)      =  uIC(3)

c     define Clairaut's invariant for this geodesic :
      glat      =  dble(uIC(1))
      dsina     =  dble(sin(uIC(3)))
c     ******************************************
      call geordG( glat,ddummy,cosl,ddummy,rpve )
c     *******************************************
      fk0       =  rpve*cosl*dsina


c     INITIALIZE O.D.E. SOLVER AT sout = s = s0 = 0.
      s         =  s0
      sout      =  s
      iflag     = +1
c     ************************************************************
      call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      nst       =  1
      if (iflag.gt.2) then
         write (0,*)' WARNING: after 1-st step: iflag= ',iflag
         write (0,*)' relerr,abserr = ',relerr,abserr
        DO 012 i=1,21
        write (0,*)' i,work : ',i,work(i)
  012   CONTINUE
        DO 014 i=1,5
        write (0,*)' i,iwork: ',i,iwork(i)
  014   CONTINUE
        write (0,*)' RE-SET to +2'
        iflag   = +2
      endif

      count = 1
      ranges(count) = s*1000.d0
      lats(count) = u(1)/torad
      longs(count) = u(2)/torad
      azis(count) = u(3) /torad
      count = count + 1
      ranges(count) = 0
c

      if (sout1.gt.0.) then
        sout      = sout1
c       ************************************************************
        call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c       ************************************************************
 
c        write(  6,510) s,(u(i)/torad,i=1,neqn),iflag
c        write(luo,510) s,(u(i)/torad,i=1,neqn)
        if (iflag.gt.2)  goto 030
      endif

c     ---------------------
  020 sout      = s + dels
c     ---------------------
      nst = nst + 1

c      if (s.gt.sfinal) then              changed  6/17/92
c     we want last point to be that of the total range, not a point past
c     this.

      if (sout.gt.sfinal) then
        sout    = sfinal
      endif

c     it is also assumed that the iflag = +2 => normal integration
c     ************************************************************
      call rkf45(fff,neqn,u,s,sout,relerr,abserr,iflag,work,iwork)
c     ************************************************************
      if (iflag.gt.2) then
         write (0,*)' WARNING: after step# ',nst,' : iflag= ',iflag
        iflin  = iflag
c       *************************************************
        call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c       *************************************************
        if (iflout.eq.9)  goto 040
        iflag  = iflout
      else
        relerr = relerr0
      endif

c     store 'em

      ranges(count) = s * 1000.d0
      lats(count) = (u(1)/torad)
      longs(count) = u(2)/torad
      azis(count) = u(3) /torad
      count = count + 1
      ranges(count) = 0

c      write(6,510) s,(u(i)/torad,i=1,neqn)




c      print 510,s,(u(i)/torad,i=1,neqn)
c      write(luo,510) s,(u(i)/torad,i=1,neqn)


      if (s.le.sfinal) then
        if (iflag.gt.2)  goto 030
        if (s.lt.sfinal) goto 020
      endif
      if (s.ge.sfinal) then
c       write( 6,510) s,(u(i)/torad,i=1,neqn),iflag  
c       use Clairaut's invariant /fk0 as check on correct azimuth
        glat      =  dble(u(1))
c       *******************************************
        call geordG( glat,ddummy,cosl,ddummy,rpve )
c       *******************************************
        azdc    = asin(fk0/(rpve*cosl))
        cosc    = cos(dble(u(3)))
        if (cosc .lt. 0.d0) then
          azdc  = dpi - azdc
        endif
        if (azdc .lt. 0.d0) then
          azdc  = 2.d0*dpi + azdc
        endif
        azic    = dble(azdc)
c       endrun diagnostic :
c        write(luo,555) s,(u(i)/torad,i=1,neqn),azic/torad

        goto 040
      endif
  030 continue
c     branching to diagnostics of abnormal performance :
      iflin = iflag
c     *************************************************
      call rkfWARN(neqn,u,s,iflin,relerr,abserr,iflout)
c     *************************************************
      if (iflout.eq.9)  goto 040
      iflag = iflout
      goto 020

  040 continue
c-      CLOSE(luo)
c      goto 010

  050 CLOSE(7)

      return
c     stop
      end


